window.Gleam.initSearch([{"doc":"based","title":"based","content":"# based\n\n[![Package Version](https://img.shields.io/hexpm/v/based)](https://hex.pm/packages/based)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/based/)\n\nThis package provides a unified interface for interacting SQL databases. With\n`based`, gleam code can remain mostly database-agnostic. SQL queries will need\nto be written for your chosen database (postgres, sqlite, etc), but execution\nof those queries isn&#39;t tied to your chosen database.\n\nPackages for different database backends can conform to the type and function\ndefinitions in this package. This can allow developers to quickly get started\nwriting gleam programs using a sqlite backend, but change to postgres when\nneeded without a large refactor.\n\n### Example packages that can be used with `based`:\n\n- [`based_pg`](https://github.com/stndrs/based_pg)\n- [`based_sqlite`](https://github.com/stndrs/based_sqlite)\n\n```sh\ngleam add based\n```\n```gleam\nimport based\nimport based_pg\n// import based_sqlite\nimport gleam/option.{None}\n\nconst sql = &quot;DELETE FROM users WHERE id=$1;&quot;\n\npub fn main() {\n  let config = load_config()\n\n  use db &lt;- based.register(based_pg.with_connection, config)\n  // use db &lt;- based.register(based_sqlite.with_connection, config)\n\n  // Swapping out the backend doesn&#39;t require rewriting the existing queries\n  based.new_query(sql)\n  |&gt; based.with_args([based.int(1)])\n  |&gt; based.exec(db)\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/based&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\ngleam shell # Run an Erlang shell\n```\n","url":"index.html"},{"doc":"based","title":"Adapter","content":"pub type Adapter(a, c) =\n  fn(Query(a), c) -&gt; Result(Returned(a), Nil)\n\n\nSynonyms:\nbased.Adapter\nbased Adapter","url":"based.html#Adapter"},{"doc":"based","title":"DB","content":"pub type DB(a, c) {\n  DB(conn: c, execute: Adapter(a, c))\n}\n\nDB(conn: c, execute: Adapter(a, c))\n\n\nSynonyms:\nbased.DB\nbased DB","url":"based.html#DB"},{"doc":"based","title":"Query","content":"pub type Query(a) {\n  Query(\n    sql: String,\n    args: List(Value),\n    decoder: Option(dynamic.Decoder(a)),\n  )\n}\n\nQuery(\n  sql: String,\n  args: List(Value),\n  decoder: Option(dynamic.Decoder(a)),\n)\n\n\nSynonyms:\nbased.Query\nbased Query","url":"based.html#Query"},{"doc":"based","title":"Returned","content":"pub type Returned(a) {\n  Returned(count: Int, rows: List(a))\n}\n\nReturned(count: Int, rows: List(a))\n\n\nSynonyms:\nbased.Returned\nbased Returned","url":"based.html#Returned"},{"doc":"based","title":"Value","content":"pub type Value {\n  String(String)\n  Int(Int)\n  Float(Float)\n  Bool(Bool)\n  Null\n}\n Callers will interact with these Value types when building queries. Their chosen\n backend is responsible for converting these Value types to the appropriate type.\nString(String)\n\n\nInt(Int)\n\n\nFloat(Float)\n\n\nBool(Bool)\n\n\nNull\n\n\nSynonyms:\nbased.Value\nbased Value","url":"based.html#Value"},{"doc":"based","title":"WithConnection","content":"pub type WithConnection(a, b, c) =\n  fn(b, fn(DB(a, c)) -&gt; Result(Returned(a), Nil)) -&gt;\n    Result(Returned(a), Nil)\n Defines a valid `with_connection` function\n\nSynonyms:\nbased.WithConnection\nbased WithConnection","url":"based.html#WithConnection"},{"doc":"based","title":"all","content":"pub fn all(\n  query: Query(a),\n  db: DB(a, b),\n) -&gt; Result(Returned(a), Nil)\n The same as `exec`, but explicitly tells a reader that all queried rows are expected.\nSynonyms:\nbased.all\nbased all","url":"based.html#all"},{"doc":"based","title":"bool","content":"pub fn bool(value: Bool) -&gt; Value\n Converts a bool to a `Value` type\nSynonyms:\nbased.bool\nbased bool","url":"based.html#bool"},{"doc":"based","title":"exec","content":"pub fn exec(\n  query: Query(a),\n  db: DB(a, b),\n) -&gt; Result(Returned(a), Nil)\n Performs the query against the provided db\nSynonyms:\nbased.exec\nbased exec","url":"based.html#exec"},{"doc":"based","title":"float","content":"pub fn float(value: Float) -&gt; Value\n Converts a float to a `Value` type\nSynonyms:\nbased.float\nbased float","url":"based.html#float"},{"doc":"based","title":"int","content":"pub fn int(value: Int) -&gt; Value\n Converts an int to a `Value` type\nSynonyms:\nbased.int\nbased int","url":"based.html#int"},{"doc":"based","title":"new_query","content":"pub fn new_query(sql: String) -&gt; Query(a)\n\nSynonyms:\nbased.new_query\nbased new_query","url":"based.html#new_query"},{"doc":"based","title":"null","content":"pub fn null() -&gt; Value\n Returns a Null Value type\nSynonyms:\nbased.null\nbased null","url":"based.html#null"},{"doc":"based","title":"one","content":"pub fn one(query: Query(a), db: DB(a, b)) -&gt; Result(a, Nil)\n Returns one queried row\nSynonyms:\nbased.one\nbased one","url":"based.html#one"},{"doc":"based","title":"register","content":"pub fn register(\n  with_connection: fn(a, fn(DB(b, c)) -&gt; Result(Returned(b), Nil)) -&gt;\n    Result(Returned(b), Nil),\n  b: a,\n  callback: fn(DB(b, c)) -&gt; Result(Returned(b), Nil),\n) -&gt; Result(Returned(b), Nil)\n Expects a `with_connection` function and its first required argument. For a library\n implementing a `with_connection` function, the required argument will likely be its\n configuration data.\n In the case of `based/testing.with_connection`, the required argument is the expected\n return data.\nSynonyms:\nbased.register\nbased register","url":"based.html#register"},{"doc":"based","title":"string","content":"pub fn string(value: String) -&gt; Value\n Converts a string to a `Value` type\nSynonyms:\nbased.string\nbased string","url":"based.html#string"},{"doc":"based","title":"with_args","content":"pub fn with_args(query: Query(a), args: List(Value)) -&gt; Query(a)\n\nSynonyms:\nbased.with_args\nbased with_args","url":"based.html#with_args"},{"doc":"based","title":"with_decoder","content":"pub fn with_decoder(\n  query: Query(a),\n  decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; Query(a)\n\nSynonyms:\nbased.with_decoder\nbased with_decoder","url":"based.html#with_decoder"},{"doc":"based","title":"based","content":"","url":"based.html"},{"doc":"based/testing","title":"Connection","content":"pub type Connection {\n  Connection(c: Nil)\n}\n A mock connection\nConnection(c: Nil)\n\n\nSynonyms:\nbased/testing.Connection\nbased/testing Connection","url":"based/testing.html#Connection"},{"doc":"based/testing","title":"adapter","content":"pub fn adapter(\n  returned: Result(Returned(a), Nil),\n) -&gt; fn(Query(a), b) -&gt; Result(Returned(a), Nil)\n A mock database adapter\nSynonyms:\nbased/testing.adapter\nbased/testing adapter","url":"based/testing.html#adapter"},{"doc":"based/testing","title":"with_connection","content":"pub fn with_connection(\n  returned: Result(Returned(a), Nil),\n  callback: fn(DB(a, Connection)) -&gt; Result(b, c),\n) -&gt; Result(b, c)\n For testing code without hitting a real database\nSynonyms:\nbased/testing.with_connection\nbased/testing with_connection","url":"based/testing.html#with_connection"},{"doc":"based/testing","title":"based/testing","content":"","url":"based/testing.html"}]);